-- ------------------------------------------------------------------------- --
-- CD-R Database Schema                                          18-SEP-2010 --
-- ------------------------------------------------------------------------- --
--
-- REMEMBER: #1 - Change the DB Version down the bottom of the file every time
--                a change is made so that automatic upgrades are possible in
--                the future.
--           #2 - Create an upgrade script to convert the previous version to
--                the new one so that the program can upgrade old database
--                versions on open.
--           #3 - Try to keep database changes _backwards compatible_ so that
--                older versions of the program can still read newer
--                databases.
--           #4 - If a change does break compatibility, remember to update the
--                minimum required program version.


PRAGMA foreign_keys=ON;

BEGIN TRANSACTION;


-- Stores information a single disc as part of a particular version of a product
CREATE TABLE Disc (
	Disc_ID INTEGER PRIMARY KEY, -- ID column managed by the database
	SeriesNumber INTEGER,        -- Series number (eg, Disc 1, Disc 2, etc)
	DiscTitle TEXT,              -- The title of the disc
	DiscText TEXT,               -- The text on the disc
	MediaType_ID INTEGER,        -- The type of disc (CD-ROM, DVD-ROM, etc)
	DiscCode TEXT,               -- A user-defined code for this particular disc
	DiscDate TEXT,               -- The date the disc was written (if known)
	Location TEXT,               -- The location where the disc is physically stored
	PictureFile TEXT,            -- The name of a file containing a picture of the disc
	ImageFileHash TEXT,          -- The hash of the disc image
	FOREIGN KEY(MediaType_ID) REFERENCES MediaType(MediaType_ID)
);

-- Type of Disk/Disc (CD, DVD, Floppy Disk, etc)
CREATE TABLE MediaType (
	MediaType_ID INTEGER PRIMARY KEY,
	Name TEXT,
	Description TEXT
);

-- Used for storing media MD5 sums. More reliable than using filenames.
CREATE TABLE Hashes (
	Sum_ID INTEGER PRIMARY KEY,
	Hash TEXT,
	Filename TEXT
);

-- Different types of metadata about this database
CREATE TABLE DBMetaDataType (
	DBMetaDataType_ID INTEGER PRIMARY KEY,
	Name TEXT
);

-- Metadata about this database
CREATE TABLE DBMetaData (
	DBMetaData_ID INTEGER PRIMARY KEY,
	DBMetaDataType_ID INTEGER,
	DataKey TEXT,
	DataValue TEXT,
	FOREIGN KEY(DBMetaDataType_ID) REFERENCES DBMetaDataType(DBMetaDataType_ID)
);

-- So MD5 sums can be looked up quickly
CREATE INDEX indHash ON Hashes(Hash);

-- Used to find all filenames for a particular Disc_ID using the MD5 sum.
CREATE VIEW qry_MainDiscView AS
	SELECT
		d.Disc_ID AS '#',
		d.DiscTitle AS 'Title',
		d.DiscCode AS 'Code',
		d.DiscDate AS 'Date',
		d.SeriesNumber AS 'Series Number',
		d.Location AS 'Storage Location',
		mt.Name AS 'Media Type'
	FROM
		Disc d
		LEFT JOIN MediaType mt ON d.MediaType_ID = mt.MediaType_ID
;

-- Used to find all filenames for a particular Disc_ID using the MD5 sum.
CREATE VIEW qry_DiscHash AS
	SELECT
		h.Hash,
		h.Filename,
		d.Disc_ID
	FROM
		Hashes h
		LEFT JOIN Disc d ON d.ImageFileHash = h.hash
;

-- Keys 1 to 50 are RESERVED for future use within this program
INSERT INTO DBMetaDataType(DBMetaDataType_ID,Name) Values(1,'DBInfo'); -- database version, minimum program version, etc
INSERT INTO DBMetaDataType(DBMetaDataType_ID,Name) Values(2,'DBProperties'); -- Author, comments, etc.
INSERT INTO DBMetaDataType(DBMetaDataType_ID,Name) Values(3,'PRGSettingsOverride'); -- For overriding program settings

-- Make a note of the database type and what created it
INSERT INTO DBMetaData(DBMetaDataType_ID,DataKey,DataValue) Values(1,'DBType','CDRIMGDB');
INSERT INTO DBMetaData(DBMetaDataType_ID,DataKey,DataValue) Values(1,'DBGenerator','schema.sql');

-- Bump this value up one every time the database format changes
INSERT INTO DBMetaData(DBMetaDataType_ID,DataKey,DataValue) Values(1,'DBVersion','1.0.0');

-- If a breaking change was introduced (fields rearranged, new required fields added, etc) bump
-- this up to the first version of the program capable of interacting correctly with the new
-- database version.
INSERT INTO DBMetaData(DBMetaDataType_ID,DataKey,DataValue) Values(1,'DBMinProgramVersion','1.0.0');

COMMIT;
