/*
-----------------------------------------------------
   Application Name: "CDRDB"
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "CDRDB".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "discdialog.h"
#include "aboutdialog.h"
#include "databaseproperties.h"
#include "mtmaintenance.h"
#include "editdata.h"
#include "version.h"

#include <QtDebug>
#include <QFileDialog>
#include <QMessageBox>
#include <QProgressDialog>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlResult>
#include <QStringList>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    qDebug() << "MainWindow Setup...";

    ui->setupUi(this);

    // Hook up actions to slots
    connect(ui->action_New_Database,SIGNAL(triggered()),this,SLOT(acnNewDB()));
    connect(ui->action_Open,SIGNAL(triggered()),this,SLOT(acnOpenDB()));
    connect(ui->action_Import_Disc_Hashes,SIGNAL(triggered()),this,SLOT(acnImportHashes()));
    connect(ui->actionE_xit,SIGNAL(triggered()),this,SLOT(acnExit()));
    connect(ui->action_New_Disc,SIGNAL(triggered()),this,SLOT(acnCreateDisc()));
    connect(ui->actionDatabase_Properties,SIGNAL(triggered()),this,SLOT(acnDBProperties()));
    connect(ui->actionMaintain_Media_Types,SIGNAL(triggered()),this,SLOT(acnMaintainMediaTypes()));
    connect(ui->action_Edit_Data,SIGNAL(triggered()),this,SLOT(acnEditData()));
    connect(ui->action_About,SIGNAL(triggered()),this,SLOT(acnAbout()));

    connect(ui->tableView,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(tvDoubleClicked(QModelIndex)));

    setActionsEnabled(false);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::acnNewDB() {
    QString filename = QFileDialog::getSaveFileName(this,"New Database");

    if (filename.isEmpty()) {
        qDebug() << "Filename is empty. Aborting.";
        return;
    }

    openDB(filename,false,false);

    QFile createQuery(":/createdb/sqlite/schema.sql");

    if (!createQuery.open(QFile::ReadOnly)) {
        QMessageBox::critical(this,"Error","Failed to open new SQLite Database Schema. Unable to create database.");
        return;
    }

    QString queryString = createQuery.readAll();
    QStringList queries = queryString.split(";");

    for (int i = 0; i < queries.count(); i++) {

        QString query = queries.at(i);
        query = query.trimmed();

        if (query.isEmpty()) continue;

        qDebug() << "Executing query: " << query;

        QSqlQuery q(query);

        QSqlError err = q.lastError();

        if (err.type() != QSqlError::NoError) {
            QMessageBox::critical(this,"Error","SQL Error " + QString::number(err.number()) + " (" + err.text() + ").");
            setActionsEnabled(false);
            database.close();
            return;
        }
    }

    QSqlQuery vu;
    vu.prepare("UPDATE DBMetaData SET DataValue='CDRDB' WHERE DataKey='DBGenerator' AND DBMetaDataType_ID=1");
    vu.exec();

    QSqlError err = vu.lastError();

    if (err.type() != QSqlError::NoError) {
        QMessageBox::critical(this,"Error","SQL Error " + QString::number(err.number()) + " (" + err.text() + ").");
        setActionsEnabled(false);
        database.close();
        return;
    }

    populateMainDisplay();
}

void MainWindow::acnOpenDB() {
    qDebug() << "MainWindow::loadDB()";
    openDB(QFileDialog::getOpenFileName(this,"Open Database"));
}

void MainWindow::acnImportHashes() {
    QMessageBox::StandardButton response =  QMessageBox::question(this, "Question","Do you wish "
                                                                  "to remove all existing Hashes "
                                                                  "from the database first?",
                                                                  QMessageBox::Yes | QMessageBox::No,
                                                                  QMessageBox::No);
    if (response == QMessageBox::Yes) {
        QSqlQuery q("DELETE FROM Hashes",database);
        if (!q.exec()) {
            QMessageBox::critical(this,"Error","Failed to delete existing Hashes from database.");
            return;
        }
    }

    QString filename = QFileDialog::getOpenFileName(this,
                                                    "Open MD5 Sum List",
                                                    "",
                                                    "MD5 files (*.md5);;All Files (*.*)");

    if (!QFileInfo(filename).exists()) {
        return;
    }

    QFile file(filename);

    if (!file.open(QFile::ReadOnly)) {
        QMessageBox::critical(this,"Error","Unable to open file.");
        return;
    }

    QProgressDialog pd("Importing...","Cancel",0,1,this);

    pd.setMaximum(0);
    pd.setMinimum(0);
    pd.setAutoClose(false);
    pd.show();

    while (!file.atEnd()) {
        QString curlin = file.readLine();
        qDebug() << curlin.trimmed();
        if (!curlin.startsWith("#") && !curlin.trimmed().isEmpty()) {
            QString hash = curlin.mid(0,32);
            QString filename = curlin.mid(34);
            filename = filename.trimmed();
            pd.setLabelText(filename);
            qDebug() << "Importing hash for file " << filename;

            // Seems QSqlQuery executes the query in its constructor.
            QSqlQuery q("INSERT INTO Hashes(Hash,Filename) VALUES('" + hash + "','" + filename + "');");
/*            if (!q.exec()) {
                QMessageBox::critical(this,"Error","Failed to insert row into database. Import aborted.");
                return;
            }
*/
        }
        QCoreApplication::processEvents();
        if (pd.wasCanceled()) {
            QMessageBox::information(this,"Canceled","Import Canceled.");
            return;
        }
    }
    pd.close();
}

void MainWindow::acnExit() {
    QCoreApplication::exit(0);
}

void MainWindow::acnCreateDisc() {
    DiscDialog *ndd = new DiscDialog(this);

    connect(ndd,SIGNAL(refreshRequested()),this,SLOT(refreshMainDisplay()));

    ndd->setSelfCleanup(true);
    ndd->newDisc();

    ndd->show();
}

void MainWindow::acnDBProperties() {
    DatabaseProperties dbp;
    dbp.exec();
}

void MainWindow::acnMaintainMediaTypes() {
    MTMaintenance mt;
    mt.exec();
}

void MainWindow::acnEditData() {
    EditData ed;
    ed.exec();
    refreshMainDisplay();
}

void MainWindow::acnAbout() {
    AboutDialog ad;
    ad.exec();
}

void MainWindow::tvDoubleClicked(const QModelIndex &index) {
    int id = queryModel.data(queryModel.index(index.row(),0)).toInt();

    DiscDialog *ndd = new DiscDialog(this);

    ndd->setSelfCleanup(true);
    ndd->showDisc(id);
    ndd->show();
}

void MainWindow::setActionsEnabled(bool enabled) {
    ui->action_Import_Disc_Hashes->setEnabled(enabled);
    ui->action_New_Disc->setEnabled(enabled);
    ui->actionDatabase_Properties->setEnabled(enabled);
    ui->actionMaintain_Media_Types->setEnabled(enabled);
    ui->action_Edit_Data->setEnabled(enabled);
}

void MainWindow::openDB(QString filename, bool typeCheck, bool refresh) {
    qDebug() << "MainWindow::openDB() - Filename:" << filename;

    if (filename.isEmpty()) {
        qDebug() << "No filename specified. Aborting.";
        return;
    }

    database = QSqlDatabase::addDatabase("QSQLITE");
    database.setDatabaseName(filename);
    if (!database.open()) {
        qDebug() << "Open database failed.";
        QMessageBox::critical(this,"Cannot open database","Database failed to open.");
        setActionsEnabled(false);
    } else {

        if (typeCheck) {
            // check version info
            QString base = "SELECT DataValue FROM DBMetaData WHERE DBMetaDataType_ID = 1 AND DataKey=";

            QSqlQuery q(base + "'DBType'");
            q.first();
            QString dbtype = q.value(0).toString();

            q.exec(base + "'DBVersion'");
            q.first();
            QString dbversion = q.value(0).toString();

            q.exec(base + "'DBMinProgramVersion'");
            q.first();
            QString dbMinVersion = q.value(0).toString();

            q.exec(base + "'DBGenerator'");
            q.first();
            QString dbGenerator = q.value(0).toString();

            qDebug() << "DBType: " << dbtype;
            qDebug() << "DBVersion: " << dbversion;
            qDebug() << "DBMinProgramVersion: " << dbMinVersion;
            qDebug() << "DBGenerator: " << dbGenerator;

            if (dbtype != "CDRIMGDB") {
                QMessageBox::critical(this,"Error","Database is not of the correct type.");
                database.close();
                setActionsEnabled(false);
                return;
            }

            // Figure out the minimum program version and see if we are new enough.
            QStringList bits = dbMinVersion.split(".");
            if (bits.length() != 3) {
                QMessageBox::warning(this,"Warning","This database has an invalid Minimum Program Version number. "
                                     "This version of CDRDB may not be properly compatible with the database "
                                     "and may malfunction.");
            } else {
                QString majorS = bits.at(0);
                const int major = majorS.toInt();
                QString minS = bits.at(1);
                const int minor = minS.toInt();
                QString revS = bits.at(2);
                const int revision = revS.toInt();

                bool tooOld = false;

                if (major > VersionInfo::major) {
                    // Major version is greater. We are out of date.
                    tooOld = true;
                } else if (major == VersionInfo::major) {
                    // Major version is the same. Onwards to the minor.
                    if (minor > VersionInfo::minor) {
                        // minor version is greater. We are out of date.
                        tooOld = true;
                    } else if (minor == VersionInfo::minor) {
                        // Major and Minor are equal. Check the revision
                        if (revision > VersionInfo::revision) {
                            tooOld = true;
                        } else {
                            tooOld = false;
                        }
                    } else {
                        // Minor is lower. We should be able to handle it.
                        tooOld = false;
                    }
                } else {
                    // Major version is lower. We should be able to handle it
                    tooOld = false;
                }

                if (tooOld) {
                    QMessageBox::critical(this,"Error","This database is for a newer version of CDRDB and is "
                                          "incompatible with this version. Please obtain a newer version of CDRDB.");
                    database.close();
                    setActionsEnabled(false);
                    return;
                }

            }
        }

        setWindowFilePath(filename);        
        if (refresh)
            populateMainDisplay();
        setActionsEnabled(true);
    }
}

void MainWindow::refreshMainDisplay() {
    populateMainDisplay();
}

void MainWindow::populateMainDisplay() {
    queryModel.setQuery("SELECT * FROM qry_MainDiscView");
    ui->tableView->setModel(&queryModel);
    ui->tableView->resizeColumnsToContents();
}
